/*
  Fork Join Example
  Demonstrates how to spawn processes and exchange messages with PVM++
*/

/* defines and prototypes for the PVM++ and PVM libraries */

#include <pvm++/pvm++.hh>

/* Maximum number of children this program will spawn */

#define MAXNCHILD   20

/* Tag to use for the join message */

#define JOINTAG     11

struct ToSendStruct:public Pvm::Struct
{
  PvmSetStructId (JOINTAG);
  PvmRegistration ()
  {
    Pvm::Register (Sender);
  }
  Pvm::Task Sender;
};

int
main (int argc, char *argv[])
{
  /* number of tasks to spawn, use 3 as the default */
  int ntask = 3;

  /* if i don't have a parent then i am the parent */
  if (!Pvm::Pvm ().I ().HasParent ())
    {
      /* find out how many tasks to spawn */
      if (argc == 2)
	ntask = atoi (argv[1]);
      /* make sure ntask is legal */
      if ((ntask < 1) || (ntask > MAXNCHILD))
	return -1;
      /* spawn the child tasks */
      Pvm::TaskSet Tasks;
      Pvm::HostSet Hosts;
      /* get all hosts in the PVM */
      Pvm::Pvm ().Hosts (Hosts);
      /* spawn the programs there */
      Hosts.Spawn (PROGNAME, ntask, Tasks);
      /* print out the task ids */
      Pvm::TaskSet::const_iterator Current;
      for (Current = Tasks.begin (); Current != Tasks.end (); ++Current)
	std::cout << *Current << ", ";
      std::cout << std::endl;

      /* as long as not all children have sent data */
      while (!Tasks.empty ())
	{
	  ToSendStruct ToReceive;
	  Pvm::Task From;
	  /* receive data from child */
	  ToReceive.Receive (From);
	  std::cout << "received data from child " << From << "." << std::endl;
	  if (From != ToReceive.Sender)
	    std::cout << "This should not happen!" << std::endl;
	  /* remove child from list of tasks to listen to */
	  Tasks.erase (From);
	}
    }
  else
    {
      ToSendStruct ToSend;
      ToSend.Sender = Pvm::Pvm ().I ();
      ToSend.Send (Pvm::Pvm ().I ().Parent ());
    }
}
