/* 
 * PVM++ - A C++ library for easy use of PVM
 * Copyright (C) 1997-2001 Sebastian Wilhelmi; University of Karlsruhe
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pvm++/private.hh"
#include <iostream>

namespace Pvm
{
  using namespace Internal;  

  void
  Throw (int Error, const char *File, int Line)
  {
    std::cerr << "Pvm Error " << Error << " occured in module " << File
	      << " on line " << Line << "." << std::endl;
    pvm_perror (NULL);
    exit (1);
  }
  
  const Class &
  Pvm ()
  {
    static Class TheClass;
    return TheClass;
  }
  
  int unsigned  
  Class::IntNumOfArchs;
  
  Class::Class ()
  {
    HostTable = new HostTableType;
    THROW (pvm_setopt (PvmAutoErr, 0));
    MyId = pvm_mytid ();
    if (MyId < 0)
      {
	THROW (pvm_start_pvmd (0, NULL, 0));
	MyId = pvm_mytid ();
	THROW (MyId);
      }
    THROW (pvm_notify (PvmHostAdd, HostAddInfo, -1, NULL));
    HostTable->Refresh ();
    ReceivedQueue = new QueueType;
    TaskTable = new TaskTableType;
    HandlerTable = new HandlerTableType;
  }
  
  Class::~Class ()
  {
    delete HandlerTable;
    delete TaskTable;
    delete ReceivedQueue;
    delete HostTable;
    pvm_exit ();
  }
  
  void
  Class::Hosts (HostSet &Result) const
  {
    Result.erase (Result.begin (), Result.end ());
    HostTableType::iterator Current = HostTable->begin ();
    HostTableType::iterator Last = HostTable->end ();
    while (Current != Last)
      {
	Result.insert ((*Current).second.Id);
	++Current;
      }
  }
  
  void
  Class::Tasks (TaskSet &Result) const
  {
    GetTasks (0, Result);
  }
  
  void
  Class::Update () const
  {
    while (true)
      {
	int Buffer = pvm_nrecv (-1, -1);
	if (Buffer == 0)
	  return;
	GetBuffer (Buffer);
      }
  }

}
