#!/bin/sh

# This shell-script generates pack.ii

# PVM++ - A C++ library for easy use of PVM
# Copyright (C) 1997-2001 Sebastian Wilhelmi; University of Karlsruhe
# 
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

FILE=pack.ii
TEMP=$FILE.$$

cat >$TEMP <<EOF
/* 
 * PVM++ - A C++ library for easy use of PVM
 * Copyright (C) 1997-2001 Sebastian Wilhelmi; University of Karlsruhe
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* This file is generated by the shell script generate_pack_ii.sh */
/* DO NOT EDIT DIRECTLY */

#ifndef __PVM_PACK_II__
#define __PVM_PACK_II__

#include <pvm++/host.hh>
#include <pvm++/hostset.hh>
#include <pvm++/task.hh>
#include <pvm++/taskset.hh>
#include <pvm++/struct.hh>

#include <cstdio>
#include <pvm3.h>
#include <string>
#include <complex>

#include <vector>
#include <list>
#include <deque>
#include <set>
#include <map>

namespace Pvm
{

/* Hack to get a non-const type out of a const one without a cast */
template< class Type > 
inline 
Type & 
UnConst (const Type &What)
{
  return const_cast< Type & > (What);
}

EOF

Template () {
echo >>$TEMP
if [ "$1" != "" ]; then
  echo "template< $1 >" >>$TEMP
fi
}

# Generate Pvm::Register for Type $1
PvmRegister () { 
Template "$2"
cat >>$TEMP <<EOF
inline
void
Register ($1 &What)
{
  if (Struct::CurrentlyPacking)
    Pack (What);
  else
    UnPack (What);
}
EOF
}

# Generate Pvm::Register for Array Type $1
PvmRegisterArray () { 
Template "$2"
cat >>$TEMP <<EOF
inline
void
Register ($1 *What, int Num)
{
  if (Struct::CurrentlyPacking)
    Pack (What, Num);
  else
    UnPack (What, Num);
}
EOF
}

PvmPack () {
Template "$3"
cat >>$TEMP <<EOF
inline
void
Pack (const $1 &What)
{
  $2
}
EOF
}

PvmUnPack () {
Template "$3"
cat >>$TEMP <<EOF
inline
void
UnPack ($1 &What)
{
  $2
}
EOF
}

PvmPackArray () {
Template "$3"
cat >>$TEMP <<EOF
inline
void
Pack ($1 *What, int Num)
{
  $2
}
EOF
}

PvmUnPackArray () {
Template "$3"
cat >>$TEMP <<EOF
inline
void
UnPack ($1 *What, int Num)
{
  $2
}
EOF
}

PvmPackArrayDefault () {
Template "$2"
cat >>$TEMP <<EOF
inline
void
Pack ($1 *What, int Num)
{
  for (int i = 0; i < Num; ++i)
    {
      Pack (*What);
      What++;
    }
}
EOF
}

PvmUnPackArrayDefault () {
Template "$2"
cat >>$TEMP <<EOF
inline
void
UnPack ($1 *What, int Num)
{
  for (int i = 0; i < Num; ++i)
    {
      UnPack (*What);
      What++;
    }
}
EOF
}

Comment () {
cat >>$TEMP <<EOF

/* $1 */
EOF
}

# Generate Pvm::Pack und Pvm::UnPack for the PVM base types.
# $1 is the C++ type name, $2 is the PVM type name, $3 is the type to
# cast the parameter to before sending/receiving (just for suppressing
# warnings)
PvmPackUnPackBase () {
PvmPack        "$1" "THROW (pvm_pk$2 (($3 *) &What, 1, 1));"
PvmUnPack      "$1" "THROW (pvm_upk$2 (($3 *) &What, 1, 1));"
PvmPackArray   "$1" "THROW (pvm_pk$2 (($3 *) What, Num, 1));"
PvmUnPackArray "$1" "THROW (pvm_upk$2 (($3 *) What, Num, 1));"
}

GenerateBase () {
Comment "$1"
PvmPackUnPackBase "$1" "$2" "$3"
PvmRegister "$1"
PvmRegisterArray "$1"
}

GenerateBase "char"               "byte"   "char"
GenerateBase "unsigned char"      "byte"   "char"
GenerateBase "short int"          "short"  "short int"
GenerateBase "int"                "int"    "int"
GenerateBase "float"              "float"  "float"
GenerateBase "double"             "double" "double"
GenerateBase "long int"           "long"   "long int"
GenerateBase "unsigned short int" "ushort" "unsigned short"
GenerateBase "unsigned int"       "uint"   "unsigned int"
GenerateBase "unsigned long int"  "ulong"  "unsigned long int"

GenerateSimpleWithOutArray () {
Comment "$1" "$4"
PvmPack "$1" "$2" "$4"
PvmUnPack "$1" "$3" "$4"
PvmRegister "$1" "$4"
}

GenerateSimple () {
Comment "$1" "$4"
PvmPack "$1" "$2" "$4"
PvmUnPack "$1" "$3" "$4"
PvmPackArrayDefault "$1" "$4"
PvmUnPackArrayDefault "$1" "$4"
PvmRegister "$1" "$4"
PvmRegisterArray "$1" "$4"
}

GenerateStlContainer () {
GenerateSimple "$1" \
 "typename $1::const_iterator Current = What.begin ();
  unsigned long int Size = What.size ();
  Pack (Size);
  while (Size > 0)
    {
      Pack (*Current);
      --Size;
      ++Current;
    }" \
 "What.erase (What.begin (), What.end ());
  std::insert_iterator< $1 > Current (What, What.begin ());
  unsigned long int Size = 0;
  UnPack (Size);
  typename $1::value_type UnPacked;
  while (Size > 0)
    {
      UnPack (UnPacked);
      *Current = UnPacked;
      --Size;
      ++Current;
    }" \
  "$2"
}

GenerateSimple "bool" \
 "Pack ((unsigned char) What);" \
 "unsigned char Value;
  UnPack (Value);
  What = Value;"

GenerateSimple "std::string" \
 "unsigned int Size = What.size ();
  THROW (pvm_pkuint (&Size, 1, 1));
  THROW (pvm_pkbyte ((char *) What.c_str (), Size, 1));" \
 "unsigned int Size;
  THROW (pvm_upkuint (&Size, 1, 1));
  char *Temp = new char[Size + 1];
  int Error = pvm_upkbyte (Temp, Size, 1);
  if (Error < 0)
    {
      delete Temp;
      THROW (Error);
    }
  Temp[Size] = '\000';
  What = Temp;
  delete Temp;"

GenerateSimple "std::complex< Type >" \
 "Pack (What.real ());
  Pack (What.imag ());" \
 "Type Real, Imag;
  UnPack (Real);
  UnPack (Imag);
  What = std::complex< Type > (Real, Imag);" \
 "class Type"
 
GenerateSimpleWithOutArray "Struct" \
 "What.Pack();" \
 "What.UnPack();"

GenerateSimpleWithOutArray "Custom" \
 "What.Pack();" \
 "What.UnPack();"

GenerateSimple "Host" \
 "unsigned int Id = What;
  THROW (pvm_pkuint (&Id, 1, 1));" \
 "unsigned int Id;
  THROW (pvm_upkuint (&Id, 1, 1));
  What = Id;"

GenerateSimple "Task" \
 "unsigned int Id = What;
  THROW (pvm_pkuint (&Id, 1, 1));" \
 "unsigned int Id;
  THROW (pvm_upkuint (&Id, 1, 1));
  What = Id;"

GenerateSimple "std::pair< First, Second >" \
 "Pack (What.first);
  Pack (What.second);" \
 "UnPack (UnConst (What.first));
  UnPack (UnConst (What.second));" \
 "class First, class Second"

GenerateStlContainer "std::vector< Type >" "class Type"
GenerateStlContainer "std::list< Type >" "class Type"
GenerateStlContainer "std::deque< Type >" "class Type"
GenerateStlContainer "std::set< Key, Compare >" "class Key, class Compare"
GenerateStlContainer "std::multiset< Key, Compare >" "class Key, class Compare"
GenerateStlContainer "std::map< Key, Value, Compare >" \
  "class Key, class Value, class Compare"
GenerateStlContainer "std::multimap< Key, Value, Compare >" \
  "class Key, class Value, class Compare"

cat >>$TEMP <<EOF
}

#endif /* __PVM_PACK_II__ */
EOF

if cmp --quiet $FILE $TEMP; then
    rm $TEMP
else
    mv $TEMP $FILE
fi
