/* 
 * PVM++ - A C++ library for easy use of PVM
 * Copyright (C) 1997-2001 Sebastian Wilhelmi; University of Karlsruhe
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pvm++/private.hh"

namespace Pvm
{
  using namespace Internal;

  ReceiveAction
  HandlerTableType::Install (Struct &Struct, ReceiveAction What)
  {  
    Pvm (); // Make sure the PVM is started.

    HandlerTableEntry FoundEntry;
    HandlerTableEntry NewEntry;
    StructId Id = Struct.GetStructId ();
    iterator Found = find (Id);
    if (Found == end ())
      {
	FoundEntry.first = ReceiveAction::Normal;
	FoundEntry.second = NULL;
      }
    else
      FoundEntry = (*Found).second;
  
    if (What.Action == ReceiveAction::Normal)
      {
	erase (Id);
	delete FoundEntry.second;
	return FoundEntry.first;
      }

    NewEntry.first = What;

    if (What.Action == ReceiveAction::UseHandler)
      NewEntry.second = Struct.GetNewInstance ();
    else if (What.Action == ReceiveAction::AutoUnpack)
      NewEntry.second = &Struct;
    else
      NewEntry.second = NULL;

    operator[](Id) = NewEntry;

    QueueIterator Current = ReceivedQueue->begin ();
    while (Current != ReceivedQueue->end ())
      {
	if ((*Current).StructId == Id)
	  {
	    int Buffer = (*Current).BufferId;
	    StructId StructId = (*Current).StructId;
	    int TaskId = (*Current).Task;
	    ReceivedQueue->erase (Current);
	    ExecuteHandler (Buffer, StructId, TaskId);
	    Current = ReceivedQueue->begin ();
	  }
	else
	  ++Current;
      }
    return FoundEntry.first;
  }

  bool
  HandlerTableType::ExecuteHandler (int Buffer, StructId Id, int TaskId)
  {
    iterator Found = find (Id);

    if (Found == end ())
      return false;

    ReceiveAction &Action = (*Found).second.first;
    Struct *TheStruct = (*Found).second.second;
    switch (Action.Action)
      {
      case ReceiveAction::UseHandler:
	{
	  Struct *New = TheStruct->GetNewInstance ();
	  THROW (pvm_setrbuf (Buffer));
	  New->UnPack ();
	  THROW (pvm_freebuf (Buffer));
	  (*Action.Func) (*New, Task (TaskId));
	  delete (New);
	  return true;
	}
      case ReceiveAction::AutoUnpack:
	{
	  THROW (pvm_setrbuf (Buffer));
	  TheStruct->UnPack ();
	  THROW (pvm_freebuf (Buffer));
	  return true;
	}
      case ReceiveAction::Swallow:
	{
	  THROW (pvm_freebuf (Buffer));
	  return true;
	}
      case ReceiveAction::Normal:
	return false;
      default:
	return false;
      }
  }
  
} // namespace Pvm
