/* 
 * PVM++ - A C++ library for easy use of PVM
 * Copyright (C) 1997-2001 Sebastian Wilhelmi; University of Karlsruhe
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pvm++/private.hh"

namespace Pvm
{
  
  void
  HostSet::Spawn (const std::string &Task, int Num, TaskSet &Result) const
  {
    Result.erase (Result.begin (), Result.end ());
    const int MaxTids = 50;
    static int Tids[MaxTids];
    HostSet Failed;
    HostSet::const_iterator Current;
    HostSet::const_iterator Last = end ();
    long int SpeedSum = 0;

    for (Current = begin (); Current != Last; ++Current)
      SpeedSum += (*Current).Speed ();

    char *TaskName = const_cast< char * > (Task.c_str ());
    while (Num > 0)
      {
	bool StartedNew = false;
	int FailedSpeed = 0;
	int Started = 0;
	int WantedToStart = 0;
	int SpeedSoFar = 0;
	int ReallyStarted = PvmNoHost;
	for (Current = begin (); Current != Last; ++Current)
	  {
	    if (!Failed.count (*Current))
	      {
		int CurrSpeed = (*Current).Speed ();
		int NumToStart = CurrSpeed *
		  (Num - WantedToStart) / (SpeedSum - SpeedSoFar);
		SpeedSoFar += CurrSpeed;
		WantedToStart += NumToStart;
		NumToStart = std::min (NumToStart, MaxTids);
		char *HostName = const_cast< char * >
		  ((*Current).Name ().c_str ());
		if (NumToStart != 0)
		  {
		    ReallyStarted = pvm_spawn (TaskName, NULL, PvmTaskHost,
					       HostName, NumToStart,
					       (int *) &Tids);
		    if (ReallyStarted <= 0)
		      {
			Failed.insert (*Current);
			FailedSpeed += CurrSpeed;
		      }
		    else
		      {
			for (int i = 0; i < ReallyStarted; ++i)
			  {
			    ::Pvm::Task Insert (Tids[i]);
			    Insert.Where->Name = TaskName;
			    Result.insert (Insert);
			  }
			Started += ReallyStarted;
			StartedNew = true;
		      }
		  }
	      }
	  }

	if (!StartedNew)
	  THROW (ReallyStarted);
	
	SpeedSum -= FailedSpeed;
	Num -= Started;
      }
  }

  Task HostSet::Spawn (const std::string &Task) const
  {
    TaskSet Which;
    Spawn (Task, 1, Which);
    return *Which.begin ();
  }
  
} // namespace Pvm
