/* 
 * PVM++ - A C++ library for easy use of PVM
 * Copyright (C) 1997-2001 Sebastian Wilhelmi; University of Karlsruhe
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pvm++/private.hh"

namespace Pvm
{
  
  HostTableType::HostTableType ()
  {
    ZeroEntry.Id = 0;
    ZeroEntry.Name = "No Host";
    ZeroEntry.Arch = "No Arch";
    ZeroEntry.Speed = 0;
    ZeroEntry.Running = false;
  }

  HostEntry *
  HostTableType::Get (unsigned int Id)
  {
    iterator Found = find (Id);

    if (Found == end ())
      return &ZeroEntry;
   
    return &(*Found).second;
  }

  void
  HostTableType::Refresh ()
  {
    int NumOfHosts;
    pvmhostinfo *Info;
    THROW (pvm_config (&NumOfHosts, (int *) &Class::IntNumOfArchs, &Info));
    int *TidArray = new int[NumOfHosts];
    for (int i = 0; i < NumOfHosts; ++i)
      {
	TidArray[i] = Info[i].hi_tid;
	HostEntry & Entry = operator[](TidArray[i]);
	Entry.Id = Info[i].hi_tid;
	Entry.Name = Info[i].hi_name;
	Entry.Arch = Info[i].hi_arch;
	Entry.Speed = Info[i].hi_speed;
	Entry.Running = true;
      }
    int Error = pvm_notify (PvmHostDelete, HostDeleteInfo,
			    NumOfHosts, TidArray);
    delete TidArray;
    THROW (Error);
  }

  void
  HostTableType::Delete (unsigned int Id)
  {
    iterator Current = find (Id);

    if (Current != end ())
      (*Current).second = ZeroEntry;
  }
  
} // namespace Pvm
