/* 
 * PVM++ - A C++ library for easy use of PVM
 * Copyright (C) 1997-2001 Sebastian Wilhelmi; University of Karlsruhe
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __PVM_INTERNAL_HH__
#define __PVM_INTERNAL_HH__

#include <pvm++/privatetypes.hh>

namespace Pvm
{

  class AccessPrivate
  {
  public:  
    static void UnPack (const QueueIterator &QueuePos, Struct &What);
    static void UnPack (const QueueIterator &QueuePos, StructSet &What,
			Task &From, StructId &StructId);
    static QueueIterator ReceiveSelect (StructSet& Set, 
					long int Sec, long int uSec);
    static bool Select (StructSet &StructSet, 
			long int EndSec, long int EnduSec);
  };

  namespace Internal
  {

    void GetTasks (int What, TaskSet & Result);
    
    void CalcEndTime (unsigned long int Time,
		      long int &Sec, long int &uSec);
    unsigned long int CalcRestTime (long int Sec, long int uSec);
    void CalcRestTime (long int Sec, long int uSec,
		       long int &dSec, long int &duSec);
    
    QueueIterator GetBuffer (int Buffer);
    QueueIterator ReceiveIt ();
    QueueIterator ReceiveIt (long int Sec, long int uSec);
    QueueIterator ReceiveItNoBlock ();
    QueueIterator ReceiveSelect (StructSet& Set, 
				 long int Sec, long int uSec);
    bool Search (const TaskSet &TaskSet,
		 const StructSet &StructSet,
		 QueueIterator &QueuePos);
    bool Search (const StructSet &StructSet,
		 QueueIterator &QueuePos);
    void UnPack (const QueueIterator &QueuePos, Struct &What);
    void UnPack (const QueueIterator &QueuePos, StructSet &What,
		 Task &From, StructId &StructId);
    bool Select (StructSet &StructSet, long int EndSec,
		 long int EnduSec);
    extern HostTableType *HostTable;
    extern TaskTableType *TaskTable;
    extern HandlerTableType *HandlerTable;
    extern QueueType *ReceivedQueue;

  } // namespace Internal

} // namespace Pvm

#endif /* __PVM_INTERNAL_HH__ */
