/* 
 * PVM++ - A C++ library for easy use of PVM
 * Copyright (C) 1997-2001 Sebastian Wilhelmi; University of Karlsruhe
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pvm++/private.hh"

namespace Pvm
{
  using namespace Internal;

  Task Struct::IgnoreTask;
  bool Struct::CurrentlyPacking;

  Struct::~Struct ()
  {  
    Pvm (); // Make sure the PVM is started.

    if (StructIdForAutomaticUnpack != -1)
      {
	HandlerTableType::iterator Found;
	Found = HandlerTable->find (StructIdForAutomaticUnpack);
	if (Found != HandlerTable->end ())
	  {
	    HandlerTableEntry FoundEntry = (*Found).second;
	    if (FoundEntry.first.Action == ReceiveAction::AutoUnpack &&
		FoundEntry.second == this)
	      HandlerTable->erase (StructIdForAutomaticUnpack);
	  }
      }
  }

  void
  Struct::Send (Task To) const
  {
    Pvm ().Update ();
    THROW (pvm_initsend (PvmDataDefault));
    Pack ();
    THROW (pvm_send (To, GetStructId () + StructIdOffset));
  }

  void
  Struct::Receive (Task &From)
  {
    StructSet Set;
    Set.insert (*this);
    Set.Receive (From);
  }

  void
  Struct::ReceiveFrom (const TaskSet &FromSet, Task &From)
  {
    StructSet Set;
    Set.insert (*this);
    Set.ReceiveFrom (FromSet, From);
  }

  void
  Struct::ReceiveFrom (Task From)
  {
    StructSet Set;
    Set.insert (*this);
    Set.ReceiveFrom (From);
  }

  bool 
  Struct::TimedReceive (unsigned long int &Time, Task &From)
  {
    StructSet Set;
    Set.insert (*this);
    return Set.TimedReceive (Time, From) != 0;
  }

  bool
  Struct::TimedReceiveFrom (const TaskSet &FromSet,
			    unsigned long int &Time, Task &From)
  {
    StructSet Set;
    Set.insert (*this);
    return Set.TimedReceiveFrom (FromSet, Time, From) != 0;
  }

  bool 
  Struct::TimedReceiveFrom (Task From, unsigned long int &Time)
  {
    StructSet Set;
    Set.insert (*this);
    return Set.TimedReceiveFrom (From, Time) != 0;
  }

  ::Pvm::ReceiveAction 
  Struct::ReceiveAction (const ::Pvm::ReceiveAction &What)
  {  
    Pvm (); // Make sure the PVM is started.

    if (What.Action == ReceiveAction::AutoUnpack)
      StructIdForAutomaticUnpack = GetStructId ();
    else
      StructIdForAutomaticUnpack = -1;
    return HandlerTable->Install (*this, What);
  }
  
} // namespace Pvm
