/* 
 * PVM++ - A C++ library for easy use of PVM
 * Copyright (C) 1997-2001 Sebastian Wilhelmi; University of Karlsruhe
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pvm++/private.hh"

namespace Pvm
{
  using namespace Internal;

  Task StructSet::IgnoreTask;

  StructId
  StructSet::Receive (Task &From)
  {  
    Pvm (); // Make sure the PVM is started.

    StructId Result;
    QueueIterator QueuePos = ReceivedQueue->begin ();
    while (true)
      {
	if (Search (*this, QueuePos))
	  {
	    UnPack (QueuePos, *this, From, Result);
	    return Result;
	  }

	QueuePos = ReceiveSelect (*this, 0, 0);
	if (QueuePos == ReceivedQueue->end ())
	  return 0;
      }
  }

  StructId
  StructSet::ReceiveFrom (const TaskSet &FromSet, Task &From)
  {  
    Pvm (); // Make sure the PVM is started.

    StructId Result;
    QueueIterator QueuePos = ReceivedQueue->begin ();
    while (true)
      {
	if (Search (FromSet, *this, QueuePos))
	  {
	    UnPack (QueuePos, *this, From, Result);
	    return Result;
	  }

	QueuePos = ReceiveSelect (*this, 0, 0);
	if (QueuePos == ReceivedQueue->end ())
	  return 0;
      }
  }

  StructId 
  StructSet::ReceiveFrom (Task From)
  {
    TaskSet FromSet;
    FromSet.insert (From);
    return ReceiveFrom (FromSet, From);
  }

  StructId
  StructSet::TimedReceive (unsigned long int &Time, Task &From)
  {  
    Pvm (); // Make sure the PVM is started.

    StructId Result;
    long int Sec, uSec;
    CalcEndTime (Time, Sec, uSec);
    while (true)
      {
	QueueIterator QueuePos = ReceivedQueue->begin ();
	if (Search (*this, QueuePos))
	  {
	    UnPack (QueuePos, *this, From, Result);
	    return Result;
	  }

	QueuePos = ReceiveSelect (*this, Sec, uSec);
	if (QueuePos == ReceivedQueue->end ())
	  return 0;
      }
  }

  StructId
  StructSet::TimedReceiveFrom (const TaskSet &FromSet,
			       unsigned long int &Time, Task &From)
  { 
    Pvm (); // Make sure the PVM is started.

    StructId Result;
    long int Sec, uSec;
    CalcEndTime (Time, Sec, uSec);
    while (true)
      {
	QueueIterator QueuePos = ReceivedQueue->begin ();
	if (Search (FromSet, *this, QueuePos))
	  {
	    UnPack (QueuePos, *this, From, Result);
	    return Result;
	  }

	QueuePos = ReceiveSelect (*this, Sec, uSec);
	if (QueuePos == ReceivedQueue->end ())
	  return 0;
      }
  }

  StructId
  StructSet::TimedReceiveFrom (Task From, unsigned long int &Time)
  {
    TaskSet FromSet;
    FromSet.insert (From);
    return TimedReceiveFrom (FromSet, Time, From);
  }
  
} // namespace Pvm
