/* 
 * PVM++ - A C++ library for easy use of PVM
 * Copyright (C) 1997-2001 Sebastian Wilhelmi; University of Karlsruhe
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pvm++/private.hh"

namespace Pvm
{
  using namespace Internal;

  bool 
  Task::HasParent () const
  {
    return Where->ParentId != 0;
  }

  Task 
  Task::Parent () const
  {
    if (Where->ParentId == 0)
      THROW (PvmNoParent);

    return Where->ParentId;
  }

  Host 
  Task::Host () const
  {
    return ::Pvm::Host (Where->HostId);
  }

  std::string 
  Task::Name () const
  {
    return Where->Name;
  }

  bool 
  Task::Running () const
  {
    if (Where->Running)
      Refresh ();
    
    return Where->Running;
  }

  Task::operator unsigned int () const
  {
    return Where->Id;
  }

  TaskEntry *
  Task::Add (unsigned int TaskId)
  {  
    Pvm (); // Make sure the PVM is started.

    return TaskTable->Get (TaskId);
  }

  void
  Task::Refresh () const
  { 
    Pvm (); // Make sure the PVM is started.

    TaskTable->Refresh (Where->Id);
  }

  void
  Task::Kill () const
  {
    if (*this == Pvm ().I ())
      {
	pvm_exit ();
	exit (0);
      }
    else if (Running ())
      THROW (pvm_kill (Where->Id));
  }

} // namespace Pvm
