/* 
 * PVM++ - A C++ library for easy use of PVM
 * Copyright (C) 1997-2001 Sebastian Wilhelmi; University of Karlsruhe
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pvm++/private.hh"

namespace Pvm
{
  
  TaskTableType::TaskTableType ()
  {
    ZeroEntry.Id = 0;
    ZeroEntry.ParentId = 0;
    ZeroEntry.HostId = 0;
    ZeroEntry.Name = "No Task";
    ZeroEntry.Running = false;
  }
  
  TaskTableType::iterator
  TaskTableType::Add (unsigned int Id, const pvmtaskinfo *What,
		      bool IsZero)
  {
    iterator Result = find (Id);
    if (Result == end ())
      Result = insert( std::make_pair( Id, ZeroEntry ) ).first;
    TaskEntry & Entry = (*Result).second;
    if (!IsZero)
      {
	Entry.ParentId = What->ti_ptid;
	Entry.Name = What->ti_a_out != NULL ? What->ti_a_out : "";
	Entry.HostId = What->ti_host;
	Entry.Running = true;
      }
    else
      Entry = ZeroEntry;
    
    Entry.Id = Id;
    return Result;
  }
  
  TaskTableType::iterator 
  TaskTableType::Refresh (unsigned int Id)
  {
    int NumOfTasks;
    pvmtaskinfo *Info;
    int Error = pvm_tasks (Id, &NumOfTasks, &Info);
    bool IsZero = Error == 
      PvmBadParam || Error == PvmNoHost || NumOfTasks == 0;
    
    if (!IsZero)
      THROW (Error);
    
    return Add (Id, Info, IsZero);
  }
  
  TaskEntry *
  TaskTableType::Get (unsigned int Id)
  {
    if (Id == 0)
      return &ZeroEntry;
    
    iterator Found = find (Id);
    
    if (Found == end ())
      Found = Refresh (Id);
    
    return &(*Found).second;
  }

} // namespace Pvm

